
##################################################
#Peitao Wu (peitaowu@bu.edu)
#Biqi Wang (wangbiqi@bu.edu)
##################################################

#####Two Continuous
#####estimating approximated beta and pvalue using full data

####data = phenotype file include two philetypes
####y2 = column names for dependent phenotype
####y1 column names for phenotype to be adjusted
####cor list of covariants
get_gamma_full_two_cont <- function(data,y2,y1,cor)
{
  if (!is.null(cor)){
    mod4<-summary(lm(as.formula(paste0(y2,'~',y1,'+',paste(cor,collapse = '+'))),data=data))
  }else{
    mod4<-summary(lm(as.formula(paste0(y2,'~',y1)),data=data))
  }
  gamma_hat<-mod4$coefficients[2,1]
  #sd_gamma_hat<-mod4$coefficients[2,2]
  return(gamma_hat)
}

#####estimating approximated beta and pvalue using subset
####data = phenotype file include two philetypes
####id = column names for subjects id
####subid = id list for subset
####y2 = column names for dependent phenotype
####y1 column names for phenotype to be adjusted
####cor list of covariants
get_gamma_subset_two_cont <- function(data,id,subid,y2,y1,cor)
{
  dd <- data[which(data[,id]%in%subid),]
  if (!is.null(cor)){
    mod4<-summary(lm(as.formula(paste0(y2,'~',y1,'+',paste(cor,collapse = '+'))),data=dd))
  }else{
    mod4<-summary(lm(as.formula(paste0(y2,'~',y1)),data=dd))
  }
  gamma_hat<-mod4$coefficients[2,1]
  #sd_gamma_hat<-mod4$coefficients[2,2]
  return(gamma_hat)
}


#####calculate approximated beta and pvalue. Before use this function, two marginal summary statisitics 
#####should be matched
#####d = Data with both two phenotypes GWAS summary statistics
#####N = sample size
#####beta_m_hat = column name for beta coeficient for SNP in Y2~SNP
#####sd_beta_m_hat = column name for standard error for beta coeficient of SNP in Y2~SNP
#####eta_hat = column name for beta coeficient for SNP in Y1~SNP
#####sd_eta = column name for standard error for beta coeficient of SNP in Y1~SNP
#####maf = column name for minor allele frequency of SNPs
#####gamma_hat = slope coefficient estimation in Y2~Y1
#####r = number of SNPs
two_con <- function(d,N,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta,maf,gamma_hat,r){
  beta_m_hat <- d[,beta_m_hat]
  sd_beta_m_hat <- d[,sd_beta_m_hat]
  eta_hat <- d[,eta_hat]
  sd_eta <- d[,sd_eta]
  maf <- d[,maf]
  xx<-2*maf*(1-maf)*N
  y2y2<-(N-1)*xx*sd_eta^2+xx*(eta_hat)^2
  y1y1<-(N-1)*xx*sd_beta_m_hat^2+xx*(beta_m_hat)^2
  xy2<-xx*eta_hat
  xy1<-xx*beta_m_hat
  beta_c<-c()
  pval_c<-c()
  var_c<-c()
  z<-c()
  for(j in 1:r)
  {
    if(is.na(xy1[j]) | is.na(xx[j]) | is.na(y2y2[j])){beta_c[j]<-NA;z[j]<-NA;pval_c[j]<-NA}else{
      v<-matrix(c(xx[j],xy2[j],xy2[j],y2y2[j]),nrow=2,ncol=2,byrow=F)
      D<-matrix(c(xx[j],0,0,y2y2[j]),nrow=2,ncol=2,byrow=F)
      temp<-solve(v)%*%D%*%c(beta_m_hat[j],gamma_hat)  
      beta_c[j]<-temp[1]
      var_c[j]<-(solve(v)*as.numeric((y1y1[j]-t(temp)%*%D%*%c(beta_m_hat[j],gamma_hat))/(N-2)))[1,1]
      z[j]<-beta_c[j]/sqrt(var_c[j])
      pval_c[j]<-2*pt(-abs(z[j]),df=N-3)
    }
  }
  
  final<-data.frame(com[1:r,],beta_c,z,pval_c)
  return(final)
}


####final function 
####method = either full, subset or literature
two_con_f <- function(data,y2,y1,cor,id,subid,d,N,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta,maf,gamma_hat,r,method)
{
  if (method =='full'){
    two_con(d,N,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta,maf,gamma_hat=get_gamma_full_two_cont(data,y2,y1,cor),r)
  }else if (method =='subset'){
    two_con(d,N,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta,maf,gamma_hat=get_gamma_subset_two_cont(data,id,subid,y2,y1,cor),r)
  }else if (method =='literature'){
    two_con(d,N,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta,maf,gamma_hat,r)
  }
}

######Example 

# library(data.table)
# dy1 <- fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/WC_adjBMI/lme2/summary2.SHARe/summary2/rawwc550k.summary2',data.table=F)
# dy2 <- fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/WC_adjBMI/lme2/summary2.SHARe/summary2/rawbmi550k.summary2',data.table=F)
##write.table(dy1[1:100,],'/restricted/projectnb/glycemic/peitao/conditional/dy1.twocon',row.names = F,col.names = T,quote = F,sep='\t')
##write.table(dy2[1:100,],'/restricted/projectnb/glycemic/peitao/conditional/dy2.twocon',row.names = F,col.names = T,quote = F,sep='\t')

# com=merge(dy1,dy2,by="SNPID")
# m3 <- fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/WC_adjBMI/wc_bmi_pheno.csv',data.table=F)
## m3 <- na.omit(m3)
### mm <- data.frame(id=1:500,wccm = rnorm(500,mean=mean(m3$wccm),sd = sd (m3$wccm)), bmi = rnorm(500,mean=mean(m3$bmi),sd = sd (m3$bmi)),sex.x = rbinom(500,1,0.5),dnaage= rnorm(500,mean(m3$dnaage),sd(m3$dnaage)) , PC1=rnorm(500,mean=mean(m3$PC1), sd =sd(m3$PC1)), PC2=rnorm(500,mean=mean(m3$PC2), sd =sd(m3$PC2)),PC3=rnorm(500,mean=mean(m3$PC3), sd =sd(m3$PC3)),PC4=rnorm(500,mean=mean(m3$PC4), sd =sd(m3$PC4)),PC5=rnorm(500,mean=mean(m3$PC5), sd =sd(m3$PC5)))
####write.table(mm,'/restricted/projectnb/glycemic/peitao/conditional/phe.twocon',row.names = F,col.names = T,quote = F,sep='\t')

# set.seed(23)
# subid <- m3[sample(1:dim(m3)[1],1000),'id']
# r1 <- two_con_f(data = m3,y2 = 'wccm',y1 = 'bmi',cor = c('dnaage','agesq','sex.x','PC1','PC2','PC3','PC4','PC5','PC6','PC7','PC8','PC9','PC10'), d = com, N = 7550, beta_m_hat='beta.x',sd_beta_m_hat='se.x',eta_hat='beta.y',sd_eta='se.y',maf='maf97.x', gamma_hat=0, r = 100, method = 'full')
# r2 <- two_con_f(data = m3,y2 = 'wccm',y1 = 'bmi',cor = c('dnaage','agesq','sex.x','PC1','PC2','PC3','PC4','PC5','PC6','PC7','PC8','PC9','PC10'),id = 'id',subid = subid, d = com, N = 7550, beta_m_hat='beta.x',sd_beta_m_hat='se.x',eta_hat='beta.y',sd_eta='se.y',maf='maf97.x', gamma_hat=0, r = 100, method = 'subset')
# r3 <- two_con_f(data = m3,y2 = 'wccm',y1 = 'bmi',cor = c('dnaage','agesq','sex.x','PC1','PC2','PC3','PC4','PC5','PC6','PC7','PC8','PC9','PC10'),id = 'id',subid = subid, d = com, N = 7550, beta_m_hat='beta.x',sd_beta_m_hat='se.x',eta_hat='beta.y',sd_eta='se.y',maf='maf97.x', gamma_hat=2.4, r = 100, method = 'literature')


