##################################################
#Peitao Wu (peitaowu@bu.edu)
#Biqi Wang (wangbiqi@bu.edu)
##################################################

####Y1 is binary and Y2 is continuous
###### function for  estimating cov(y1,y2) and variance of smoking
#####d = Data with both two phenotypes GWAS summary statistics
#####eta_hat = column name for beta coeficient for SNP in Y1~SNP
#####maf = column name for minor allele frequency of SNPs
#####N = sample size
#####Ncase = number of cases in Y1
cov_est <- function(d,eta_hat,maf, N, Ncase){
  Nctrl<-N-Ncase
  prev<-Ncase/N
  r<-nrow(d)
  eta_hat<-d[,eta_hat]
  maf<-d[,maf]
  
  p0<-c()
  p1<-c()
  ## a is the OR of SNP x on Y2, b is the proportion of non-cases(controls), c is the proportion of cases, p is the allele frequency of x
  s<-function(a,b,c,p)
  {
    p1[1]<-(-b+p-a*p-a*c+sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2))/(2*(c-a*c))
    p1[2]<-(-b+p-a*p-a*c-sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2))/(2*(c-a*c))
    p0[1]<-(-a*p+p-sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2)+a*c+b)/(2*b*(1-a))
    p0[2]<-(-a*p+p+sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2)+a*c+b)/(2*b*(1-a))
    return(c(p0,p1))
  }
  u<-c()
  for (j in 1:r){
    temp<-s(exp(eta_hat[j]),1-prev,prev,maf[j])
    u<-rbind(u,temp)
  }
  u<-u[,c(1,3)]
  a<-c()
  b<-c()
  c<-c()
  d<-c()
  e<-c()
  f<-c()
  for(j in 1:r)
  {
    a[j]<-(1-u[j,1])^2*Nctrl
    c[j]<-2*(1-u[j,1])*u[j,1]*Nctrl
    e[j]<-u[j,1]^2*Nctrl
    
    b[j]<-(1-u[j,2])^2*Ncase
    d[j]<-2*(1-u[j,2])*u[j,2]*Ncase
    f[j]<-u[j,2]^2*Ncase
  }
  est<-cbind(a,c,e,b,d,f)
  est_cov<-c()
  
  for(j in 1:r)
  {
    est_cov[j]<-1/N*(est[j,1]*(2*maf[j])*(prev)+est[j,2]*(1-2*maf[j])*(0-prev)+est[j,3]*(2-2*maf[j])*(0-prev)+est[j,4]*(0-2*maf[j])*(1-prev)
                     +est[j,5]*(1-2*maf[j])*(1-prev)+est[j,6]*(2-2*maf[j])*(1-prev))
  }
  return(est_cov)
}

###example 

# dy1<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/fhs550k/lme2/summary2.SHARe/summary2/bmi550kgwas.summary2',data.table=F)
# dy2<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/fhs550k/geepack/summary2.SHARe/summary2/smk550kgwas.summary2',data.table=F)
# data<-merge(dy1,dy2,by="SNPID")
# data<-data[1:100,]
# est_cov <- cov_est(d = data,eta_hat = 'beta.y',maf = 'maf97.x', N = 6330, Ncase = 3334)

# Estimate the regression coefficient of y2~y1, full
####data = phenotype file include two philetypes
####y2 = column names for dependent phenotype
####y1 column names for phenotype to be adjusted
####cor list of covariants
get_gamma_full_y1_bi <- function(data,y2,y1,cor)
{ 
  if (!is.null(cor)){
  mod4<-summary(lm(as.formula(paste0(y2,'~',y1,'+',paste(cor,collapse = '+'))),data=data))
}else{
  mod4<-summary(lm(as.formula(paste0(y2,'~',y1)),data=data))
}
  gamma_hat<-mod4$coefficients[2,1]
  #sd_gamma_hat<-mod4$coefficients[2,2]
  return(gamma_hat)
}

#####estimating approximated beta and pvalue using subset
####data = phenotype file include two philetypes
####id = column names for subjects id
####subid = id list for subset
####y2 = column names for dependent phenotype
####y1 column names for phenotype to be adjusted
####cor list of covariants
get_gamma_subset_y1_bi <- function(data,id,subid,y2,y1,cor)
{
  dd <- data[which(data[,id]%in%subid),]
  if (!is.null(cor)){
    mod4<-summary(lm(as.formula(paste0(y2,'~',y1,'+',paste(cor,collapse = '+'))),data=dd))
  }else{
    mod4<-summary(lm(as.formula(paste0(y2,'~',y1)),data=dd))
  }
  gamma_hat<-mod4$coefficients[2,1]
  #sd_gamma_hat<-mod4$coefficients[2,2]
  return(gamma_hat)
}


####example
# pheno<-fread("/restricted/projectnb/sequencing/biqiwang/cond_phenotype/pheno_gwas.csv",data.table=F)
# get_gamma_full_y1_bi(data=pheno,y2='resbmi',y1 = 'ever_smk', cor = NULL)
# set.seed(23)
# subid=pheno[sample(1:dim(pheno)[1],1000),'id']
# get_gamma_subset_y1_bi(data=pheno,id ='id',subid=subid ,y2='resbmi',y1 = 'ever_smk', cor = NULL)

#####calculate approximated beta and pvalue. Before use this function, two marginal summary statisitics 
#####should be matched
#####d = Data with both two phenotypes GWAS summary statistics
#####n_m4 = column names for sample size for SNPs
#####b_m1 = column name for beta coeficient for SNP in Y2~SNP
#####se_m1 = column name for standard error for beta coeficient of SNP in Y2~SNP
#####eta_hat = column name for beta coeficient for SNP in Y1~SNP
#####maf = column name for minor allele frequency of SNPs
#####N = sample size
#####Ncase = number of cases in Y1
#####gamma = slope coefficient estimation in Y2~Y1

y1_bi <- function(d,n_m4,b_m1,se_m1,eta_hat,maf,N,Ncase,gamma){
  r <- nrow(d)
  b_m1<-d[,b_m1]
  n_m4<-d[,n_m4]
  se_m1<-d[,se_m1]
  prev<-Ncase/N
  est_cov <- cov_est (d, eta_hat, maf, N, Ncase)
  maf<-d[,maf]
  beta_c<-c()
  sigma_j<-c()
  se_c<-c()
  z<-c()
  pval_c<-c()
  
  for(j in 1:r)
  {
    if(is.na(est_cov[j]) | is.na(maf[j])){beta_c[j]<-NA;se_c[j]<-NA;z[j]<-NA;pval_c[j]<-NA}else{
      v<-matrix(c(2*maf[j]*(1-maf[j]),est_cov[j],est_cov[j],prev*(1-prev)),nrow=2,ncol=2,byrow=F)
      D<-matrix(c(2*maf[j]*(1-maf[j]),0,0,prev*(1-prev)),nrow=2,ncol=2,byrow=F)
      v_i<-solve(v)
      temp<-v_i%*%D%*%c(b_m1[j],gamma)  
      beta_c[j]<-temp[1]
      sigma_j[j]<-(((se_m1[j]^2*(n_m4[j]-1)+b_m1[j]^2)*2*maf[j]*(1-maf[j])*(n_m4[j]-1))-t(temp)%*%D%*%c(b_m1[j],gamma))/(n_m4[j]-2)
      se_c[j]<-sqrt((sigma_j[j]*v_i)[1,1]/(n_m4[j]-1))
      z[j]<-beta_c[j]/se_c[j]
      pval_c[j]<-2*pt(-abs(z[j]),df=n_m4[j]-3)
    }
  }
  
  
  final<-data.frame(d,beta_c,se_c,z,pval_c)
  return(final)
}

#####example
# data$n_m4<-data$n0.x+data$n1.x+data$n2.x
# pheno<-fread("/restricted/projectnb/sequencing/biqiwang/cond_phenotype/pheno_gwas.csv",data.table=F)
# gamma <- get_gamma_full_y1_bi(data=pheno,y2='resbmi',y1 = 'ever_smk', cor = NULL)
# r1<-y1_bi(d=data,n_m4='n_m4',b_m1='beta.x',se_m1='se.x',eta_hat = 'beta.y',maf = 'maf97.x', N = 6330, Ncase = 3334,gamma = gamma)

# set.seed(23)
# subid=pheno[sample(1:dim(pheno)[1],1000),'id']
# gamma <- get_gamma_subset_y1_bi(data=pheno,id ='id',subid=subid ,y2='resbmi',y1 = 'ever_smk', cor = NULL)

# r2<-y1_bi(d=data,n_m4='n_m4',b_m1='beta.x',se_m1='se.x',eta_hat = 'beta.y',maf = 'maf97.x', N = 6330, Ncase = 3334,gamma = gamma)

######final function

####final function 
####method = either full, subset or literature
y1_bi_f <- function(d,n_m4,b_m1,se_m1,eta_hat,maf,N,Ncase,gamma,data,id,subid,y2,y1, cor, method)
{
  if (method =='full'){
    y1_bi(d,n_m4,b_m1,se_m1,eta_hat,maf,N,Ncase,gamma = get_gamma_full_y1_bi(data,y2,y1, cor))
    }else if (method =='subset'){
    y1_bi(d,n_m4,b_m1,se_m1,eta_hat,maf,N,Ncase,gamma = get_gamma_subset_y1_bi(data,id,subid, y2, y1, cor))
  }else if (method =='literature'){
    y1_bi(d,n_m4,b_m1,se_m1,eta_hat,maf,N,Ncase,gamma)
  }
}
#####example
 # dy1<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/fhs550k/lme2/summary2.SHARe/summary2/bmi550kgwas.summary2',data.table=F)
 # dy2<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/fhs550k/geepack/summary2.SHARe/summary2/smk550kgwas.summary2',data.table=F)

##write.table(dy1[1:100,],'/restricted/projectnb/glycemic/peitao/conditional/dy1.y1bi',row.names = F,col.names = T,quote = F,sep='\t')
##write.table(dy2[1:100,],'/restricted/projectnb/glycemic/peitao/conditional/dy2.y1bi',row.names = F,col.names = T,quote = F,sep='\t')

 # data<-merge(dy1,dy2,by="SNPID")
 # data<-data[1:100,]
 # data$n_m4<-data$n0.x+data$n1.x+data$n2.x
 # pheno<-fread("/restricted/projectnb/sequencing/biqiwang/cond_phenotype/pheno_gwas.csv",data.table=F)
## pheno <- na.omit(pheno)
### mm <- data.frame(id=1:500,resbmi = rnorm(500,mean=mean(pheno$resbmi),sd = sd (pheno$resbmi)), ever_smk  = rbinom(500,1,0.527) ,sex.x = rbinom(500,1,0.5),dnaage= rnorm(500,mean(pheno$dnaage),sd(pheno$dnaage)) , PC1=rnorm(500,mean=mean(pheno$PC1), sd =sd(pheno$PC1)), PC2=rnorm(500,mean=mean(pheno$PC2), sd =sd(pheno$PC2)),PC3=rnorm(500,mean=mean(pheno$PC3), sd =sd(pheno$PC3)),PC4=rnorm(500,mean=mean(pheno$PC4), sd =sd(pheno$PC4)),PC5=rnorm(500,mean=mean(pheno$PC5), sd =sd(pheno$PC5)))
####write.table(mm,'/restricted/projectnb/glycemic/peitao/conditional/phe.y1bi',row.names = F,col.names = T,quote = F,sep='\t')

 # r1<-y1_bi_f(d=data,n_m4='n_m4',b_m1='beta.x',se_m1='se.x',eta_hat = 'beta.y',maf = 'maf97.x', N = 6330, Ncase = 3334,gamma = 0, data = pheno, id = 'id', subid=NULL ,y2='resbmi',y1 = 'ever_smk', cor = NULL,method='full')
 # set.seed(23)
 # subid=pheno[sample(1:dim(pheno)[1],1000),'id']
 # r2<-y1_bi_f(d=data,n_m4='n_m4',b_m1='beta.x',se_m1='se.x',eta_hat = 'beta.y',maf = 'maf97.x', N = 6330, Ncase = 3334,gamma = 0, data = pheno, id = 'id', subid=subid ,y2='resbmi',y1 = 'ever_smk', cor = NULL,method='subset')
 