##################################################
#Peitao Wu (peitaowu@bu.edu)
#Biqi Wang (wangbiqi@bu.edu)
##################################################


####Y1 is continuous and Y2 is binary

###### function for  estimating cov(y1,y2) and variance of smoking
#####d = Data with both two phenotypes GWAS summary statistics
#####beta_m_hat = column name for beta coeficient for SNP in Y2~SNP
#####maf = column name for minor allele frequency of SNPs
#####N = sample size
#####Ncase = number of cases in Y2
cov_est <- function(d,beta_m_hat,maf, N, Ncase){
  Nctrl<-N-Ncase
  prev<-Ncase/N
  r<-nrow(d)
  beta_m_hat<-d[,beta_m_hat]
  maf<-d[,maf]
  
  p0<-c()
  p1<-c()
  ## a is the OR of SNP x on Y2, b is the proportion of non-cases(controls), c is the proportion of cases, p is the allele frequency of x
  s<-function(a,b,c,p)
  {
    p1[1]<-(-b+p-a*p-a*c+sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2))/(2*(c-a*c))
    p1[2]<-(-b+p-a*p-a*c-sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2))/(2*(c-a*c))
    p0[1]<-(-a*p+p-sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2)+a*c+b)/(2*b*(1-a))
    p0[2]<-(-a*p+p+sqrt(a^2*p^2-2*a^2*c*p+a^2*c^2+2*a*b*p+2*a*b*c-2*a*p^2+2*a*c*p+b^2-2*b*p+p^2)+a*c+b)/(2*b*(1-a))
    return(c(p0,p1))
  }
  u<-c()
  for (j in 1:r){
    temp<-s(exp(beta_m_hat[j]),1-prev,prev,maf[j])
    u<-rbind(u,temp)
  }
  u<-u[,c(1,3)]
  a<-c()
  b<-c()
  c<-c()
  d<-c()
  e<-c()
  f<-c()
  for(j in 1:r)
  {
    a[j]<-(1-u[j,1])^2*Nctrl
    c[j]<-2*(1-u[j,1])*u[j,1]*Nctrl
    e[j]<-u[j,1]^2*Nctrl
    
    b[j]<-(1-u[j,2])^2*Ncase
    d[j]<-2*(1-u[j,2])*u[j,2]*Ncase
    f[j]<-u[j,2]^2*Ncase
  }
  est<-cbind(a,c,e,b,d,f)
  est_cov<-c()
  
  for(j in 1:r)
  {
    est_cov[j]<-1/N*(est[j,1]*(2*maf[j])*(prev)+est[j,2]*(1-2*maf[j])*(0-prev)+est[j,3]*(2-2*maf[j])*(0-prev)+est[j,4]*(0-2*maf[j])*(1-prev)
                     +est[j,5]*(1-2*maf[j])*(1-prev)+est[j,6]*(2-2*maf[j])*(1-prev))
  }
  return(est_cov)
}


# m1<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/prevAFcov.summary2.commonv',data.table=F)
# m2<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/fhs_hrc_gwas/lme2/summary2.HRC.r1.1.gwas.2016/summary2/hgtcm.summary2.commonv',data.table=F)
# m3<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/afprev.cov.2014.csv',data.table=F)
# names(m1)<-c("Chr","SNPID","Phys_location","phen","N","iMAF","variMAF","Nd","iMAFd","variMAFd","beta","se","chisq","df","model","remark","pval","effect","noneffect","obs_eaf","ratio","input_snp","iwgadiff","iwgafold","inrefgene","refgenes_60kb_from_SNP","closestrefgene","distancefromclosestrefgene")
# names(m2)<-c("Chr","SNPID","Phys_location","phen","N","iMAF","variMAF","h2q","beta","se","chisq","df","model","pval","effect","noneffect","obs_eaf","ratio","input_snp","iwgadiff","iwgafold","inrefgene","refgenes_60kb_from_SNP","closestrefgene","distancefromclosestrefgene")
# com<-merge(m1,m2,by="SNPID")
# com <- com[1:100,]
# r<-nrow(com)
# t <- cov_est(d = com, beta_m_hat = 'beta.x', maf = 'iMAF.x', N = 8466, Ncase = 319)




# Estimate the regression coefficient of y2~y1, full
####data = phenotype file include two philetypes
####y2 = column names for dependent phenotype
####y1 column names for phenotype to be adjusted
####cor list of covariants
get_gamma_full_y2_bi <- function(data,y2,y1,cor)
{ 
  if (!is.null(cor)){
    mod4<-summary(glm(as.formula(paste0(y2,'~',y1,'+',paste(cor,collapse = '+'))),data=data,family=binomial))
  }else{
    mod4<-summary(glm(as.formula(paste0(y2,'~',y1)),data=data, family=binomial))
  }
  gamma_hat<-mod4$coefficients[2,1]
  sd_gamma_hat<-mod4$coefficients[2,2]
  return(c(gamma_hat,sd_gamma_hat))
}

#####estimating approximated beta and pvalue using subset
####data = phenotype file include two philetypes
####id = column names for subjects id
####subid = id list for subset
####y2 = column names for dependent phenotype
####y1 column names for phenotype to be adjusted
####cor list of covariants
get_gamma_subset_y2_bi <- function(data,id,subid,y2,y1,cor)
{
  dd <- data[which(data[,id]%in%subid),]
  if (!is.null(cor)){
    mod4<-summary(glm(as.formula(paste0(y2,'~',y1,'+',paste(cor,collapse = '+'))),data=dd, family=binomial))
  }else{
    mod4<-summary(glm(as.formula(paste0(y2,'~',y1)),data=dd, family=binomial))
  }
  gamma_hat<-mod4$coefficients[2,1]
  sd_gamma_hat<-mod4$coefficients[2,2]
  return(c(gamma_hat,sd_gamma_hat))
}

# m3<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/afprev.cov.2014.csv',data.table=F)
# mod4<-summary(glm(m3$prevAF~m3$hgtcm+m3$dnaage+m3$sex.x,family=binomial))
# gamma_hat<-mod4$coefficients[2,1]
# sd_gamma_hat<-mod4$coefficients[2,2]
# get_gamma_full_y2_bi(data =m3,y2 = 'prevAF',y1 = 'hgtcm',cor =c('dnaage','sex.x'))
# set.seed(23)
# sub<-m3[sample(1:dim(m3)[1],1000),'id']
# get_gamma_subset_y2_bi(data = m3, id ='id',subid= sub,y2 = 'prevAF', y1 = 'hgtcm',cor =c('dnaage','sex.x'))
#   
  


#####should be matched
#####d = Data with both two phenotypes GWAS summary statistics
#####N = sample size for Y2
#####N2 = sample size for Y1
#####beta_m_hat = column name for beta coeficient for SNP in Y2~SNP
#####sd_beta_m_hat = column name for standard error for beta coeficient of SNP in Y2~SNP
#####eta_hat = column name for beta coeficient for SNP in Y1~SNP
#####sd_eta = column name for standard error for beta coeficient of SNP in Y1~SNP
#####maf = column name for minor allele frequency of SNPs
#####gamma_hat = slope coefficient estimation in Y2~Y1
#####mean1 = mean of y1 when Y2 = 1
#####mean0 = mean of y1 when Y2 = 0

y2_bi <- function(d,N2,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta,maf,N,Ncase,gamma_hat,sd_gamma_hat,mean1,mean0)
{
  r <- nrow(d)
  est_cov <- cov_est(d, beta_m_hat, maf, N, Ncase)
  prev<-Ncase/N
  xy1<-est_cov*N
  beta_m_hat<-d[,beta_m_hat]
  sd_beta_m_hat<-d[,sd_beta_m_hat]
  eta_hat<-d[,eta_hat]
  sd_eta<-d[,sd_eta]
  maf<-d[,maf]
  xx<-2*maf*(1-maf)*N
  y2y2<-(N2-1)*xx*sd_eta^2+xx*(eta_hat)^2
  y1y1<-N*prev*(1-prev)
  xy2<-xx*eta_hat
  y1y2<-N*prev*(1-prev)*(mean1-mean0)
  
  var_beta_m_hat<-sd_beta_m_hat^2
  var_gamma_hat<-sd_gamma_hat^2
  beta_c<-c()
  pval_c<-c()
  var_c<-c()
  z<-c()
  
  for(j in 1:r)
  {
    if(is.na(xy1[j]) | is.na(xx[j]) | is.na(y2y2[j])){beta_c[j]<-NA;z[j]<-NA;pval_c[j]<-NA}else{
      v<-matrix(c(xx[j],xy2[j],xy2[j],y2y2[j]),nrow=2,ncol=2,byrow=F)
      D<-matrix(c(xx[j],0,0,y2y2[j]),nrow=2,ncol=2,byrow=F)
      temp<-solve(v)%*%D%*%c(beta_m_hat[j],gamma_hat)	
      beta_c[j]<-temp[1]
      var_c[j]<-(solve(v)%*%D%*%matrix(c(var_beta_m_hat[j],y1y1/N*eta_hat[j]*(y2y2[j])^(-1)*sd_beta_m_hat[j]*sd_gamma_hat/(sqrt((xx)^(-1)/(N-1)*(y1y1-xy1[j]/xx*xy1[j])*(y2y2[j])^(-1)/(N-1)*(y1y1-y1y2/y2y2[j]*y1y2))),y1y1/N*eta_hat[j]*(y2y2[j])^(-1)*sd_beta_m_hat[j]*sd_gamma_hat/(sqrt((xx)^(-1)/(N-1)*(y1y1-xy1[j]/xx*xy1[j])*(y2y2[j])^(-1)/(N-1)*(y1y1-y1y2/y2y2[j]*y1y2))),var_gamma_hat),nrow=2,ncol=2,byrow=F)%*%D%*%solve(v))[1,1]
      z[j]<-beta_c[j]/sqrt(var_c[j])
      pval_c[j]<-2*pt(-abs(z[j]),df=N-3)
    }
  }
  
  final<-data.frame(d,beta_c,z,pval_c)
  
}

#####example

# m1<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/prevAFcov.summary2.commonv',data.table=F)
# m2<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/fhs_hrc_gwas/lme2/summary2.HRC.r1.1.gwas.2016/summary2/hgtcm.summary2.commonv',data.table=F)
# m3<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/afprev.cov.2014.csv',data.table=F)
# names(m1)<-c("Chr","SNPID","Phys_location","phen","N","iMAF","variMAF","Nd","iMAFd","variMAFd","beta","se","chisq","df","model","remark","pval","effect","noneffect","obs_eaf","ratio","input_snp","iwgadiff","iwgafold","inrefgene","refgenes_60kb_from_SNP","closestrefgene","distancefromclosestrefgene")
# names(m2)<-c("Chr","SNPID","Phys_location","phen","N","iMAF","variMAF","h2q","beta","se","chisq","df","model","pval","effect","noneffect","obs_eaf","ratio","input_snp","iwgadiff","iwgafold","inrefgene","refgenes_60kb_from_SNP","closestrefgene","distancefromclosestrefgene")
# com<-merge(m1,m2,by="SNPID")
# com <- com[1:100,]
# gamma <- get_gamma_full_y2_bi(data =m3,y2 = 'prevAF',y1 = 'hgtcm',cor =c('dnaage','sex.x'))
# mean1 <- mean(m3$hgtcm[m3$prevAF==1], na.rm =T)
# mean0 <- mean(m3$hgtcm[m3$prevAF==0], na.rm =T)
# t <- y2_bi (d=com,N2=8168,beta_m_hat= 'beta.x',sd_beta_m_hat = 'se.x',eta_hat = 'beta.y',sd_eta= 'se.y', maf= 'iMAF.x',N = 8466,Ncase = 319,gamma_hat = gamma[1],sd_gamma_hat = gamma[2],mean1 =mean1,mean0 = mean0)
#  set.seed(23)
#  sub<-sample(1:dim(m3)[1],1000)
#  mm<-m3[sub,]
#  subid<-m3[sub,'id']
#  mean1 <- mean(mm$hgtcm[mm$prevAF==1], na.rm =T)
#  mean0 <- mean(mm$hgtcm[mm$prevAF==0], na.rm =T)
#  gamma <- get_gamma_subset_y2_bi(data = m3, id ='id',subid= subid,y2 = 'prevAF', y1 = 'hgtcm',cor =c('dnaage','sex.x'))
#  t2 <- y2_bi (d=com,N2=8168,beta_m_hat= 'beta.x',sd_beta_m_hat = 'se.x',eta_hat = 'beta.y',sd_eta= 'se.y', maf= 'iMAF.x',N = 8466,Ncase = 319,gamma_hat = gamma[1],sd_gamma_hat = gamma[2],mean1 =mean1,mean0 = mean0)


#####final fuction 
y2_bi_f <- function (d,N2,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta,maf,N,Ncase,gamma_hat,sd_gamma_hat,mean1,mean0,data,id,subid,y2,y1,cor,method)
{
  if (method == 'full'){
    y2_bi (d,N2,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta, maf,N,Ncase, gamma_hat = get_gamma_full_y2_bi(data,y2,y1,cor)[1] ,sd_gamma_hat = get_gamma_full_y2_bi(data,y2,y1,cor)[2], mean1 = mean(data[data[,y2]==1,y1],na.rm=T), mean0 = mean(data[data[,y2]==0,y1],na.rm=T) )
  }else if (method == 'subset'){
    ds <- data[which(data[,id]%in%subid),]
    y2_bi (d,N2,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta, maf,N,Ncase, gamma_hat = get_gamma_subset_y2_bi(data , id ,subid,y2, y1,cor)[1],sd_gamma_hat = get_gamma_subset_y2_bi(data , id ,subid,y2, y1,cor)[2],mean1 =mean(ds[ds[,y2]==1,y1],na.rm=T),mean0 = mean(ds[ds[,y2]==0,y1],na.rm=T))
    }else if (method == 'literature'){
    y2_bi (d,N2,beta_m_hat,sd_beta_m_hat,eta_hat,sd_eta, maf,N,Ncase, gamma_hat,sd_gamma_hat,mean1,mean0)
   }
}


######example

# m1<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/prevAFcov.summary2.commonv',data.table=F)
# m2<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/fhs_hrc_gwas/lme2/summary2.HRC.r1.1.gwas.2016/summary2/hgtcm.summary2.commonv',data.table=F)
# m3<-fread('/restricted/projectnb/sequencing/biqiwang/cond_phenotype/afprev.cov.2014.csv',data.table=F)
# names(m1)<-c("Chr","SNPID","Phys_location","phen","N","iMAF","variMAF","Nd","iMAFd","variMAFd","beta","se","chisq","df","model","remark","pval","effect","noneffect","obs_eaf","ratio","input_snp","iwgadiff","iwgafold","inrefgene","refgenes_60kb_from_SNP","closestrefgene","distancefromclosestrefgene")
# names(m2)<-c("Chr","SNPID","Phys_location","phen","N","iMAF","variMAF","h2q","beta","se","chisq","df","model","pval","effect","noneffect","obs_eaf","ratio","input_snp","iwgadiff","iwgafold","inrefgene","refgenes_60kb_from_SNP","closestrefgene","distancefromclosestrefgene")
####
##write.table(m1[1:100,],'/restricted/projectnb/glycemic/peitao/conditional/dy1.y2bi',row.names = F,col.names = T,quote = F,sep='\t')
##write.table(m2[1:100,],'/restricted/projectnb/glycemic/peitao/conditional/dy2.y2bi',row.names = F,col.names = T,quote = F,sep='\t')
### mm <- data.frame(id=1:500,hgtcm = rnorm(500,mean=mean(m3$hgtcm),sd = sd (m3$hgtcm)), prevAF  = rbinom(500,1,0.05) ,sex.x = rbinom(500,1,0.5),dnaage= rnorm(500,mean(m3$dnaage),sd(m3$dnaage)))
####write.table(mm,'/restricted/projectnb/glycemic/peitao/conditional/phe.y2bi',row.names = F,col.names = T,quote = F,sep='\t')

# com<-merge(m1,m2,by="SNPID")
# com <- com[1:100,]
# 
# t1 <- y2_bi_f(d=com,N2=8168,beta_m_hat= 'beta.x',sd_beta_m_hat = 'se.x',eta_hat = 'beta.y',sd_eta= 'se.y', maf= 'iMAF.x',N = 8466,Ncase = 319,gamma_hat = 0,sd_gamma_hat = 0,mean1 =0,mean0 = 0,data =m3,y2 = 'prevAF',y1 = 'hgtcm',cor =c('dnaage','sex.x'), method='full')
#  set.seed(23)
#  sub<-sample(1:dim(m3)[1],1000)
#  subid<-m3[sub,'id']
# t2 <- y2_bi_f(d=com,N2=8168,beta_m_hat= 'beta.x',sd_beta_m_hat = 'se.x',eta_hat = 'beta.y',sd_eta= 'se.y', maf= 'iMAF.x',N = 8466,Ncase = 319,gamma_hat = 0,sd_gamma_hat = 0,mean1 =0,mean0 = 0,data =m3,y2 = 'prevAF',y1 = 'hgtcm',cor =c('dnaage','sex.x'), method='subset',id='id',subid=subid)
# 
